/*******************************************************
*
*	SM3D 3D point template
*
********************************************************
*	SM3D
*
*	File type	:	Template
*	Author		:	Dario Deledda
*	Date		:	25/11/2000
*	Last update	:	25/11/2000
*	Version		:	0.9a
*
********************************************************/
#ifndef SM3DPOINTTEMP
#define SM3DPOINTTEMP

#include "../header/SM3DTypes.h"
#include <math.h>

#ifdef WIN32
#define smPointT smPoint<class X>
#else
#define smPointT smPoint
#endif

template <class X> class smPoint{
public:
	X x,y,z;
	
	smPoint(X xi=0, X yi=0, X zi=0) { x=xi;y=yi;z=zi;}
	
	int pippo();
	smPoint	operator+(smPoint op2);
	smPoint	operator-(smPoint op2);
	X		operator*(smPoint op2);
	smPoint	operator/(smPoint op2);

	smPoint	operator=(smPoint op2);

	smPoint	operator+(X op2);
	smPoint	operator*(X op2);
	smPoint	operator=(X op2);

 	smPoint	ruota_zv(smPoint pr, UInt16 ga);
 	smPoint	ruota_z(smPoint pr, Fl32 ga);
 	smPoint	ruota_yv(smPoint pr, UInt16 ga);
 	smPoint	ruota_y(smPoint pr, Fl32 be);
 	smPoint	ruota_xv(smPoint pr, UInt16 ga);
 	smPoint	ruota_x(smPoint pr, Fl32 al);

	X		maxC();					//	max value between x,y,z components
	X		minC();					//	min value between x,y,z components

	X		mod();					//	module in abs value

	X		dist(smPoint op2);		//	distance from another point
};


template<class X>   smPointT  smPointT::operator+(register smPointT op2){
	register smPoint tmp;
	tmp.x=x+op2.x;
	tmp.y=y+op2.y;
	tmp.z=z+op2.z;
	return tmp;
}

template<class X>   smPointT  smPointT::operator-(register smPointT op2){
	register smPoint tmp;
	tmp.x=x-op2.x;
	tmp.y=y-op2.y;
	tmp.z=z-op2.z;
	return tmp;
}

template<class X>   X  smPointT::operator*(register smPointT op2){
	register X tmp;
	tmp=(x*op2.x + y*op2.y + z*op2.z);
	return tmp;
}

template<class X>   smPointT  smPointT::operator/(register smPointT op2){
	register smPoint tmp;
	tmp.x=y*op2.z-z*op2.y;
	tmp.y=-(x*op2.z-z*op2.x);
	tmp.z=x*op2.y-y*op2.x;
	return tmp;
}

template<class X>   smPointT  smPointT::operator=(register smPointT op2){
	x=op2.x;
	y=op2.y;
	z=op2.z;
	return *this;
}

template<class X>   smPointT  smPointT::operator=(register X op2){
	x=op2;
	y=op2;
	z=op2;
	return *this;
}

template<class X>	smPointT  smPointT::operator+(register X op2){
	register smPoint tmp;
	tmp.x=x+op2;
	tmp.y=y+op2;
	tmp.z=z+op2;
	return tmp;
}

template<class X>	smPointT  smPointT::operator*(register X op2){
	register smPoint tmp;
	tmp.x=x*op2;
	tmp.y=y*op2;
	tmp.z=z*op2;
	return tmp;
}

template<class X>   X  smPointT::maxC(){
	register X m;
	m=absv(x);
	m=((m > absv(y))? m: absv(y));
	m=((m > absv(z))? m: absv(z));
	return m;
}

template<class X>   X  smPointT::minC(){
	register X m;
	m=absv(x);
	if (m==0) m=0.00001;
	m=((m < absv(y))? m: absv(y));
	m=((m < absv(z))? m: absv(z));
	return m;
}

template<class X>   X  smPointT::mod(){
	return sqrt(x*x+y*y+z*z);
}

template<class X>   X  smPointT::dist(register smPointT op2){
	register smPoint tmp;
	tmp.x=x-op2.x;
	tmp.y=y-op2.y;
	tmp.z=z-op2.z;
	return (tmp.x*tmp.x + tmp.x*tmp.y + tmp.z*tmp.z);
}

template<class X> smPointT smPointT::ruota_zv(smPointT pr, UInt16 ga){
	register smPoint tmp,pu;
	register Int32 cs,sn;
	cs=cosv[ga];
	sn=sinv[ga];
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.y=(tmp.y*cs+tmp.x*sn)>>Ks_b;
	pu.x=(-tmp.y*sn+tmp.x*cs)>>Ks_b;

	tmp.x=pu.x+pr.x;
	tmp.y=pu.y+pr.y;
	tmp.z=tmp.z+pr.z;
	return tmp;
}

template<class X> smPointT smPointT::ruota_z(smPointT pr, Fl32 ga){
	register smPoint tmp,pu;
	register Fl32 cs,sn;
	cs=cos(ga);
	sn=sin(ga);
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.y=tmp.y*cs+tmp.x*sn;
	pu.x=-tmp.y*sn+tmp.x*cs;

	tmp.x=pu.x+pr.x;
	tmp.y=pu.y+pr.y;
	tmp.z=tmp.z+pr.z;
	return tmp;
}

template<class X> smPointT smPointT::ruota_yv(smPointT pr, UInt16 be){
	register smPoint tmp,pu;
	register Int32 cs,sn;
	cs=cosv[be];
	sn=sinv[be];
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.z=(tmp.z*cs+tmp.x*sn)>>Ks_b;
	pu.x=(-tmp.z*sn+tmp.x*cs)>>Ks_b;

	tmp.x=pu.x+pr.x;
	tmp.y=tmp.y+pr.y;
	tmp.z=pu.z+pr.z;
	return tmp;
}

template<class X> smPointT smPointT::ruota_y(smPointT pr, Fl32 be){
	register smPoint tmp,pu;
	register Fl32 cs,sn;
	cs=cos(be);
	sn=sin(be);
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.z=tmp.z*cs+tmp.x*sn;
	pu.x=-tmp.z*sn+tmp.x*cs;

	tmp.x=pu.x+pr.x;
	tmp.y=tmp.y+pr.y;
	tmp.z=pu.z+pr.z;
	return tmp;
}

template<class X> smPointT smPointT::ruota_xv(smPointT pr, UInt16 al){
	register smPoint tmp,pu;
	register Int32 cs,sn;
	cs=cosv[al];
	sn=sinv[al];
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.y=(tmp.y*cs+tmp.z*sn)>>Ks_b;
	pu.z=(-tmp.y*sn+tmp.z*cs)>>Ks_b;

	tmp.x=tmp.x+pr.x;
	tmp.y=pu.y+pr.y;
	tmp.z=pu.z+pr.z;
	return tmp;
}
template<class X> smPointT smPointT::ruota_x(smPointT pr, Fl32 al){
	register smPoint tmp,pu;
	register Fl32 cs,sn;
	cs=cos(al);
	sn=sin(al);
	tmp.x=x-pr.x;
	tmp.y=y-pr.y;
	tmp.z=z-pr.z;
	
	pu.y=tmp.y*cs+tmp.z*sn;
	pu.z=-tmp.y*sn+tmp.z*cs;

	tmp.x=tmp.x+pr.x;
	tmp.y=pu.y+pr.y;
	tmp.z=pu.z+pr.z;
	return tmp;
}
#endif