/*******************************************************
*
*	SM3D Utility routine
*
********************************************************
*	SM3D
*
*	File type	:	Source CPP
*	Author		:	Dario Deledda
*	Date		:	25/11/2000
*	Last update	:	25/11/2000
*	Version		:	0.9a
*
********************************************************/
#define SM3DMAT
#include "../header/SM3DCamera.h"
#include "../header/SM3DUtility.h"
#include <math.h>
#undef SM3DMAT

Fl32 	fcos[Ncmp],fsin[Ncmp];			// seno e coseno per operazioni veloci float
Int32	cosv[Ncmp],sinv[Ncmp];			// seno e coseno per operazioni veloci int
Int8  	col32[256*5];					// tabelle per il calcolo dell' illuminazione  32 bit
UInt16 	col16r[32*5],col16g[32*5],col16b[32*5];

UInt32	*z_buf=0;			// puntatore al buffer per tutte le camere
UInt32	frame=0;			// conteggio dei frame stampati a video
UInt32	frame1=0;

UInt32	smVBuffer=0x6000000;

void SM3DInit(void){
	SM3DInitLight();
	SM3DInitMath();
	init_Light();
}

void SM3DShutDown(void){
	del_Light();
}

void SM3DInitLight(void){
	register Int32 t;

	for(t=0; t<255; t++) {
		col32[t]=0;			// inizializzazione tabella 32 bit
		col32[t+255]=0;
		col32[t+512]=t;
		col32[t+768]=255; 
		col32[t+1024]=255; 
	}

	for (t=0; t<32; t++) {
		col16b[t]=0;			// inizializzazione tabella 16 bit
		col16b[t+32]=0;
		col16b[t+64]=t;
		col16b[t+96]=31; 
		col16b[t+128]=31; 
		
		col16g[t]=0;			
		col16g[t+32]=0;
		col16g[t+64]=t<<5;
		col16g[t+96]=31<<5; 
		col16g[t+128]=31<<5;
		
		col16r[t]=0;			
		col16r[t+32]=0;
		col16r[t+64]=t<<10;
		col16r[t+96]=31<<10; 
		col16r[t+128]=31<<10; 
	 }
}

void SM3DInitMath(void){
	register Fl32 t;
	register Int32 q;
	for(q=0;q<Ncmp;q++){
		t=q*(2*pi/Ncmp);
		fsin[q]=sin(t);
		fcos[q]=cos(t);
		sinv[q]=(int)(sin(t)*Ks_n);
		cosv[q]=(int)(cos(t)*Ks_n);
	}
}

Int32 Ang(Int32 tmp){
	if(abs(tmp) > Ncmp)
		tmp= tmp % Ncmp;
	if(tmp<0)
		tmp=Ncmp+tmp;
	return tmp;
}

#define sm2piNcmp (Ncmp/pi2)

Int32 RadToNcmp(Fl32 ang){
	Int32 tmp;
	if(ang < 0.0f)
		ang=-fmod((float)ang,(float)pi2);
	else
		ang=fmod((float)ang,(float)pi2);
		
	tmp = (Int32)(ang*sm2piNcmp);
	if(abs(tmp) > Ncmp)
		tmp= tmp % Ncmp;
	if(tmp<0)
		tmp=Ncmp+tmp;
	return tmp;
}

#ifdef WIN32

void smPutPixel(UInt8 x,UInt8 y,smRGB16 ci){}
void smCls(){}
void smBuffer::Cls(UInt16 buf){}
void smBuffer::smPutPixel(UInt8 x,UInt8 y,smRGB16 ci){}

#else

void smPutPixel(UInt8 x,UInt8 y,smRGB16 ci){
//	Pnt Buffer=(Pnt)0x6000000;
	if(x>=0 && x<smHWVwidth && y>=0 && y<smHWVheight){
	 __asm {
	         mov    r0, y
	         mov    r1,#320
	         mul    r2,r0,r1
	         mov    r1,x
	         add    r2,r2,r1
	         add    r2,r2,r1
			 mov    r0,#0x6000000
//	         mov    r0,Buffer
	         add    r0,r0,r2
	         mov    r1,ci.cl
	         strh   r1,[r0],1
	      }
	}
}

void smCls(){
	__asm{
		mov		r0,#0x6000000
		mov		r1,#(160*128)/2
		mov		r2,#0
		mov		r3,#1
	loop:
		str		r2,[r0],4
		sub		r1,r1,r3
		cmp		r1,#0
		bne		loop
	}
}

void smBuffer::Cls(UInt16 buf){
	Pnt	actBuffer=mBBuffer;

	if(buf)
		actBuffer=mFBuffer;
	
	__asm{
		mov		r0,actBuffer
		mov		r1,#(160*128)/2
		mov		r2,#0
		mov		r3,#1
	loop:
		str		r2,[r0],4
		sub		r1,r1,r3
		cmp		r1,#0
		bne		loop
	}
}


void smBuffer::smPutPixel(UInt8 x,UInt8 y,smRGB16 ci){
	Pnt	actBuffer=mBBuffer;
	if(x>=0 && x<smHWVwidth && y>=0 && y<smHWVheight){
	 __asm {
	         mov    r0, y
	         mov    r1,#320
	         mul    r2,r0,r1
	         mov    r1,x
	         add    r2,r2,r1
	         add    r2,r2,r1
	         mov    r0,actBuffer
	         add    r0,r0,r2
	         mov    r1,ci.cl
	         strh   r1,[r0],1
	      }
	}
}

#endif

void smBuffer::Clear(UInt32 in){
	++frame;
	frame1=frame<<24;
}

smBuffer::smBuffer(){
	Cls(smBACK);
	Cls(smFRONT);
#ifndef WIN32
	*(UInt16*)0x4000000=0x15;
#endif
	mFrame=0;
	mBBuffer=(Pnt)smHWGB5_Buf1;
	mFBuffer=(Pnt)smHWGB5_Buf2;
	if(!z_buf){
		z_buf=new UInt32[smHWVwidth*smHWVheight];
	}
}

smBuffer::~smBuffer(){
	if(z_buf){
		delete[] z_buf;
	}
}

void smBuffer::Swap(UInt16 sync){
#ifndef WIN32
		if(sync){
			__asm
			{
				mov 	r0, #0x4000006
			loop:
				ldrh	r1, [r0]
				cmp	r1, #128
				bne 	loop
			}
		}
		if(mFrame & 0x1){
			*(UInt16*)0x4000000=0x5;
			mBBuffer=(Pnt)smHWGB5_Buf2;
			mFBuffer=(Pnt)smHWGB5_Buf1;
		} else {
			*(UInt16*)0x4000000=0x15;
			mBBuffer=(Pnt)smHWGB5_Buf1;
			mFBuffer=(Pnt)smHWGB5_Buf2;
		}
#endif
		mFrame++;
		frame1=mFrame<<24;
			
}

Pnt	smBuffer::GetBackBuffer(){
	return mBBuffer;
}

void smBuffer::SetBackBuffer(Pnt in){
	mBBuffer=in;
}

Pnt	smBuffer::GetFrontBuffer(){
	return mFBuffer;
}

void smBuffer::SetFrontBuffer(Pnt in){
	mFBuffer=in;
}

//------------------------------------------------------------------------------
//	Draw a line in the grid
//------------------------------------------------------------------------------
void smBuffer::smLine(Int32 p1x, Int32 p1y, Int32 p2x, Int32 p2y, smRGB16 col){	
	Int32 dx,dy,sx,sy;
	Int32 x,y;
	Int32 e,i;
	Int32 pox,poy;
	Pnt	actBuffer=mBBuffer;
	bool change=false;
	
	if(p1x<0 && p2x<0) return;
	if(p1x>smHWVwidth && p2x>smHWVwidth) return;
	
	if(p1y<0 && p2y<0) return;
	if(p1y>smHWVheight && p2y>smHWVheight) return;
	
	if(p1x==p2x && p1y==p2y) return;
	
	if(p1x<0) p1x=0;
	if(p1x>smHWVwidth) p1x=smHWVwidth;
	if(p2x<0) p2x=0;
	if(p2x>smHWVwidth) p2x=smHWVwidth;

	if(p1y<0) p1y=0;
	if(p1y>smHWVheight) p1y=smHWVheight;
	if(p2y<0) p2y=0;
	if(p2y>smHWVheight) p2y=smHWVheight;
	
	x=p1x;
	y=p1y;
	dx=abs(p2x-p1x);
	dy=abs(p2y-p1y);
	
	sx=(p2x-p1x);
	sy=(p2y-p1y);

	if(sx>0) sx=1;
	else if(sx<0) sx=-1;
	else sx=0;
	
	if(sy>0) sy=1;
	else if(sy<0) sy=-1;
	else sy=0;
			
	if( dy > dx){
		swap(dx,dy);
		change = true;
	}
	
	e=2*dy-dx;
	
	for(i=0; i <= dx ; i++){
		pox=x;
		poy=y;
#ifndef WIN32		
		if(x>=0 && x<smHWVwidth && y>=0 && y<smHWVheight){
		 __asm {
		         mov    r0, poy
		         mov    r1,#320
		         mul    r2,r0,r1
		         mov    r1,pox
		         add    r2,r2,r1
		         add    r2,r2,r1
		         mov    r0,actBuffer
		         add    r0,r0,r2
		         mov    r1,col.cl
		         strh   r1,[r0],1
		      }
		}
#endif

		while(e > 0){
			if(change){
				x += sx;
			}else{
				y += sy;
			}
			e=e-(dx<<1);
		}
	
		if(change){
			y +=sy;
		}else{
			x +=sx;
		}
		e=e+(dy<<1);
	}	
}
